@@ -29,6 +29,7 @@ from typing import Any, Iterable, List, Mapping, NamedTuple, Optional, Union
 import numpy as np
 
 from google.protobuf import descriptor
+from google.protobuf import message
 # resources dependency
 # pylint: disable=unused-import
 # pylint: enable=unused-import
@@ -120,6 +121,8 @@ NAME_TO_TYPE: Mapping[str, '_PacketDataType'] = {
         _PacketDataType.PROTO,
     '::mediapipe::ClassificationList':
         _PacketDataType.PROTO,
+    '::mediapipe::ClassificationListCollection':
+        _PacketDataType.PROTO,
     '::mediapipe::Detection':
         _PacketDataType.PROTO,
     '::mediapipe::DetectionList':
@@ -128,6 +131,8 @@ NAME_TO_TYPE: Mapping[str, '_PacketDataType'] = {
         _PacketDataType.PROTO,
     '::mediapipe::LandmarkList':
         _PacketDataType.PROTO,
+    '::mediapipe::LandmarkListCollection':
+        _PacketDataType.PROTO,
     '::mediapipe::NormalizedLandmark':
         _PacketDataType.PROTO,
     '::mediapipe::FrameAnnotation':
@@ -140,6 +145,8 @@ NAME_TO_TYPE: Mapping[str, '_PacketDataType'] = {
         _PacketDataType.PROTO,
     '::mediapipe::NormalizedLandmarkList':
         _PacketDataType.PROTO,
+    '::mediapipe::NormalizedLandmarkListCollection':
+        _PacketDataType.PROTO,
     '::mediapipe::Image':
         _PacketDataType.IMAGE,
     '::std::vector<::mediapipe::Classification>':
@@ -217,6 +224,7 @@ class SolutionBase:
         calculator_params is not allowed to be modified.
         e) If the calculator options field is a repeated field but the field
         value to be set is not iterable.
+        f) If not all calculator params are valid.
     """
     if bool(binary_graph_path) == bool(graph_config):
       raise ValueError(
@@ -247,27 +255,29 @@ class SolutionBase:
     for stream_name in self._output_stream_type_info.keys():
       self._graph.observe_output_stream(stream_name, callback, True)
 
-    input_side_packets = {
+    self._input_side_packets = {
         name: self._make_packet(self._side_input_type_info[name], data)
         for name, data in (side_inputs or {}).items()
     }
-    self._graph.start_run(input_side_packets)
+    self._graph.start_run(self._input_side_packets)
 
   # TODO: Use "inspect.Parameter" to fetch the input argument names and
   # types from "_input_stream_type_info" and then auto generate the process
   # method signature by "inspect.Signature" in __init__.
   def process(
-      self, input_data: Union[np.ndarray, Mapping[str,
-                                                  np.ndarray]]) -> NamedTuple:
+      self, input_data: Union[np.ndarray, Mapping[str, Union[np.ndarray,
+                                                             message.Message]]]
+  ) -> NamedTuple:
     """Processes a set of RGB image data and output SolutionOutputs.
 
     Args:
       input_data: Either a single numpy ndarray object representing the solo
-        image input of a graph or a mapping from the stream name to the image
-        data that represents every input streams of a graph.
+        image input of a graph or a mapping from the stream name to the image or
+        proto data that represents every input streams of a graph.
 
     Raises:
-      NotImplementedError: If input_data contains non image data.
+      NotImplementedError: If input_data contains audio data or a list of proto
+        objects.
       RuntimeError: If the underlying graph occurs any error.
       ValueError: If the input image data is not three channel RGB.
 
@@ -300,8 +310,15 @@ class SolutionBase:
     self._simulated_timestamp += 33333
     for stream_name, data in input_dict.items():
       input_stream_type = self._input_stream_type_info[stream_name]
-      if (input_stream_type == _PacketDataType.IMAGE_FRAME or
-          input_stream_type == _PacketDataType.IMAGE):
+      if (input_stream_type == _PacketDataType.PROTO_LIST or
+          input_stream_type == _PacketDataType.AUDIO):
+        # TODO: Support audio data.
+        raise NotImplementedError(
+            f'SolutionBase can only process non-audio and non-proto-list data. '
+            f'{self._input_stream_type_info[stream_name].name} '
+            f'type is not supported yet.')
+      elif (input_stream_type == _PacketDataType.IMAGE_FRAME or
+            input_stream_type == _PacketDataType.IMAGE):
         if data.shape[2] != RGB_CHANNELS:
           raise ValueError('Input image must contain three channel rgb data.')
         self._graph.add_packet_to_input_stream(
@@ -309,11 +326,10 @@ class SolutionBase:
             packet=self._make_packet(input_stream_type,
                                      data).at(self._simulated_timestamp))
       else:
-        # TODO: Support audio data.
-        raise NotImplementedError(
-            f'SolutionBase can only process image data. '
-            f'{self._input_stream_type_info[stream_name].name} '
-            f'type is not supported yet.')
+        self._graph.add_packet_to_input_stream(
+            stream=stream_name,
+            packet=self._make_packet(input_stream_type,
+                                     data).at(self._simulated_timestamp))
 
     self._graph.wait_until_idle()
     # Create a NamedTuple object where the field names are mapping to the graph
@@ -338,6 +354,12 @@ class SolutionBase:
     self._input_stream_type_info = None
     self._output_stream_type_info = None
 
+  def reset(self) -> None:
+    """Resets the graph for another run."""
+    if self._graph:
+      self._graph.close()
+      self._graph.start_run(self._input_side_packets)
+
   def _initialize_graph_interface(
       self,
       validated_graph: validated_graph_config.ValidatedGraphConfig,
@@ -419,7 +441,7 @@ class SolutionBase:
         else:
           field_label = calculator_options.DESCRIPTOR.fields_by_name[
               field_name].label
-          if field_label is descriptor.FieldDescriptor.LABEL_REPEATED:
+          if field_label == descriptor.FieldDescriptor.LABEL_REPEATED:
             if not isinstance(field_value, Iterable):
               raise ValueError(
                   f'{field_name} is a repeated proto field but the value '
@@ -478,6 +500,8 @@ class SolutionBase:
       # have been visited.
       if num_modified == len(nested_calculator_params):
         break
+    if num_modified < len(nested_calculator_params):
+      raise ValueError('Not all calculator params are valid.')
 
   def _make_packet(self, packet_data_type: _PacketDataType,
                    data: Any) -> packet.Packet:
